/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.lang;

import org.apithefire.util.lang.reflect.InvocationException;

/**
 * Utility class related to {@link Class}.
 * 
 * @author <a href="mailto:bachew@gmail.com">Chew Boon Aik</a>
 */
public class Classes {

	private Classes() {
	}

	/**
	 * Creates a new instance from {@code clazz}.
	 * 
	 * @throws RuntimeIllegalAccessException
	 *             if the class or its nullary constructor is not accessible.
	 * @throws RuntimeInstantiationException
	 *             if this Class represents an abstract class, an interface, an
	 *             array class, a primitive type, or void; or if the class has
	 *             no nullary constructor; or if the instantiation fails for
	 *             some other reason.
	 * @throws ExceptionInInitializerError
	 *             if the initialization invoked by this method fails.
	 * @throws SecurityException
	 *             if access is denied by security manager
	 * @throws InvocationException
	 *             if the constructor being invoked throws exception.
	 */
	public static <T> T newInstance(Class<T> clazz) {
		try {
			return clazz.newInstance();
		} catch (IllegalAccessException e) {
			throw new RuntimeIllegalAccessException(e);
		} catch (InstantiationException e) {
			throw new RuntimeInstantiationException(e);
		} catch (ExceptionInInitializerError e) {
			throw e;
		} catch (SecurityException e) {
			throw e;
		} catch (Throwable e) {
			throw new InvocationException(e);
		}
	}

}
