/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.lang;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

/**
 * An immutable list of string components used to represent paths that are
 * commonly encoded as strings.
 * 
 * @author <a href="mailto:bachew@gmail.com">Chew Boon Aik</a>
 */
public class Path implements Comparable<Path> {
	
	private static class PathComparator implements Comparator<Path> {
		
		private final Comparator<String> componentComparator;
		
		public PathComparator() {
			componentComparator = new Comparator<String>() {
				public int compare(String string1, String string2) {
					return string1.compareTo(string2);
				}
			};
		}
		
		public PathComparator(Comparator<String> componentComparator) {
			this.componentComparator = Objects.nonNull(componentComparator); 
		}

		public int compare(Path path1, Path path2) {
			for (int i = 0; i < path1.components.length
					&& i < path2.components.length; ++i) {
				
				int diff = componentComparator.compare(path1.components[i],
						path2.components[i]);

				if (diff != 0) {
					return diff;
				}
			}
			return path1.components.length - path2.components.length;
		}
		
	}
	
	public static final Path EMPTY_PATH = new Path();
	
	private static final String SEPARATOR_SLASH = "/";
	private static final String SEPARATOR_DOT = ".";
	private static final Comparator<Path> DEFAULT_COMPARATOR = createComprator();
	
	private final String[] components;
	private String defaultString;
	
	/**
	 * Constructs an empty path.
	 */
	public Path() {
		components = new String[0];
	}
	
	private Path(int length) {
		components = new String[length];
	}
	
	/**
	 * Constructs a single component path using the specified component.
	 */
	public Path(String component) {
		components = new String[] { Objects.nonNull(component) };
	}
	
	/**
	 * Constructs a path with the specified components.
	 */
	public Path(String... components) {
		this.components = new String[components.length];
		
		for (int i = 0; i < this.components.length; ++i) {
			this.components[i] = Objects.nonNull(components[i]);
		}
	}

	/**
	 * Constructs a path with the specified components.
	 */
	public Path(List<String> components) {
		this.components = new String[components.size()];

		for (int i = 0; i < this.components.length; ++i) {
			this.components[i] = Objects.nonNull(components.get(i));
		}
	}
	
	private static void throwEmptySeparatorArgument() {
		throw new IllegalArgumentException("Path separator is empty.");
	}
	
	/**
	 * Splits slash separated of the specified string into a {@link Path}.
	 * 
	 * @return the split string
	 */
	public static Path split(String str) {
		return split(str, SEPARATOR_SLASH);
	}
	
	/**
	 * Splits dot separated name of the specified package into a {@link Path}.
	 * 
	 * @return the split package name
	 */
	public static Path split(Package pkg) {
		String name = pkg.getName();
		
		if (name == null) {
			return Path.EMPTY_PATH;
		}
		return split(name, SEPARATOR_DOT);
	}
	
	/**
	 * Splits {@code string} into a {@link Path} with the specified separator.
	 * 
	 * @throws IllegalArgumentException
	 *             if separator is empty.
	 */
	public static Path split(String string, String separator) {
		if (separator.length() == 0) {
			throwEmptySeparatorArgument();
		}
		
		int start = 0;
		int end = string.indexOf(separator);
		List<String> components = new ArrayList<String>();
		
		String component = null;
	
		while (end != -1) {
			component = string.substring(start, end);
			components.add(component);
			start = end + separator.length();
			end = string.indexOf(separator, start);
		}
		component = string.substring(start);
		components.add(component);
		return new Path(components);
	}
	
	/**
	 * Returns true if this path is empty, false otherwise.
	 */
	public boolean isEmpty() {
		return components.length < 1;
	}
	
	/**
	 * Returns the number of components.
	 */
	public int length() {
		return components.length;
	}
	
	/**
	 * Returns the component at specified index.
	 * 
	 * @throws IndexOutOfBoundsException
	 *             if {@code index} is negative or not less than then
	 *             {@linkplain #length} of this path.
	 */
	public String componentAt(int index) {
		return components[index];
	}
	
	/**
	 * Returns a copy of this path with leading and trailing empty components
	 * trimmed.
	 */
	public Path trim() {
		return trim(true, true);
	}

	/**
	 * Returns a copy of this path with leading empty components trimmed.
	 */
	public Path trimLeft() {
		return trim(true, false);
	}

	/**
	 * Returns a copy of this path with trailing empty components trimmed.
	 */
	public Path trimRight() {
		return trim(false, true);
	}
	
	private Path trim(boolean trimLeft, boolean trimRight) {
		if (components.length == 0) {
			return new Path();
		}
		
		int left = 0;
		int right = components.length;
		
		if (trimLeft) {
			while (left < components.length && components[left].length() == 0) {
				++left;
			}
		}
		if (trimRight) {
			while (right > 0 && components[right - 1].length() == 0) {
				--right;
			}
		}
		if (left > right) {
			return new Path();
		}
		
		Path path = new Path(right - left);
		
		for (int i = left; i < right; ++i) {
			path.components[i - left] = components[i];
		}
		return path;
	}
	
	/**
	 * Returns a concatenated path of this path and the specified path.
	 */
	public Path concat(Path path) {
		if (path.isEmpty()) {
			return this;
		}
		
		int concatLength = components.length + path.components.length;
		Path concatPath = new Path(concatLength);
		
		for (int i = 0; i < components.length; ++i) {
			concatPath.components[i] = components[i];
		}
		for (int i = 0; i < path.components.length; ++i) {
			concatPath.components[components.length + i] = path.components[i];
		}
		return concatPath;
	}
	
	/**
	 * Returns a concatenated path of this path and the specified component.
	 */
	public Path concat(String component) {
		Path path = new Path(length() + 1);
		
		for (int i = 0; i < components.length; ++i) {
			path.components[i] = components[i];
		}
		path.components[components.length] = Objects.nonNull(component);
		return path;
	}
	
	/**
	 * Returns sub path of this path from specified component index to the end
	 * of this path.
	 * 
	 * @throws IndexOutOfBoundsException
	 *             if {@code beginIndex} is negative or not less than
	 *             {@linkplain #length} of this path.
	 */
	public Path subPath(int beginIndex) {
		return subPath(beginIndex, components.length);
	}
	
	/**
	 * Returns sub path of this path with range {@[beginIndex, endIndex)}.
	 * 
	 * @throws IllegalArgumentException
	 *             if {@code beginIndex} is greater than {@code endIndex}.
	 * @throws IndexOutOfBoundsException
	 *             if {@code beginIndex} is negative or {@code endIndex} is not
	 *             less than {@linkplain #length} of this path.
	 */
	public Path subPath(int beginIndex, int endIndex) {
		if (beginIndex > endIndex) {
			throw new IllegalArgumentException(
					"beginIndex is greater than endIndex.");
		}
		
		Path subPath = new Path(endIndex - beginIndex);
		
		for (int i = beginIndex; i < endIndex; ++i) {
			subPath.components[i - beginIndex] = components[i];
		}
		return subPath;
	}

	/**
	 * Returns the index within this path of the first occurrence of the
	 * specified component or {@code -1} if this path does not contains the
	 * specified component.
	 */
	public int indexOf(String component) {
		Objects.nonNull(component);
		
		for (int i = 0; i < components.length; ++i) {
			if (components[i].equals(component)) {
				return i;
			}
		}
		return -1;
	}
	
	/**
	 * Returns the index within this path of the first occurrence of the
	 * specified path or {@code -1} if this path does not contains the specified
	 * sub path.
	 */
	public int indexOf(Path path) {
		return indexOf(components, path.components);
	}
	
	private static int indexOf(String[] c1, String[] c2) {
		if (c1.length < c2.length) {
			return -1;
		}
		if (c1.length == 0 && c2.length == 0) {
			return 0;
		}
		for (int i = 0; i < c1.length - c2.length + 1; ++i) {
			if (startsWith(c1, i, c2)) {
				return i;
			}
		}
		return -1;
	}
	
	protected static boolean startsWith(String[] c1, int c1Begin, String[] c2) {
		for (int c1Index = c1Begin; c1Index < c1.length; ++c1Index) {
			int c2Index = 0;
			
			while (c2Index < c2.length
					&& c1Index + c2Index < c1.length
					&& c1[c1Index + c2Index].equals(c2[c2Index])) {
				++c2Index;
			}
			return c2Index == c2.length;
		}
		return c2.length == 0;
	}
	
	/**
	 * Returns true if this path starts with the specified component, false
	 * otherwise.
	 */
	public boolean startsWith(String component) {
		return indexOf(component) == 0;
	}
	
	/**
	 * Returns true if this path starts with the specified path, false
	 * otherwise.
	 */
	public boolean startsWith(Path path) {
		return indexOf(path) == 0;
	}

	/**
	 * Returns the index within this path of the last occurrence of the
	 * specified component or {@code -1} if this path does not contains the
	 * specified component.
	 */
	public int lastIndexOf(String component) {
		Objects.nonNull(component);
		
		for (int i = components.length - 1; i >= 0; --i) {
			if (components[i].equals(component)) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Returns the index within this path of the last occurrence of the
	 * specified path or {@code -1} if this path does not contains the specified
	 * sub path.
	 */
	public int lastIndexOf(Path path) {
		return lastIndexOf(components, path.components);
	}
	
	private static int lastIndexOf(String[] c1, String[] c2) {
		if (c1.length < c2.length) {
			return -1;
		}
		if (c1.length == 0 && c2.length == 0) {
			return 0;
		}
		for (int i = c1.length - c2.length; i >= 0; --i) {
			if (startsWith(c1, i, c2)) {
				return i;
			}
		}
		return -1;
	}
	
	/**
	 * Returns true if this path ends with the specified component, false
	 * otherwise.
	 */
	public boolean endsWith(String component) {
		int index = lastIndexOf(component);
		
		return index == -1 ? false : index == components.length - 1;
	}
	
	/**
	 * Returns true if this path ends with the specified path, false otherwise.
	 */
	public boolean endsWith(Path path) {
		int index = lastIndexOf(path);
		
		if (index == -1) {
			return false;
		}
		return index == components.length - path.components.length;
	}
	
	/**
	 * Returns true if this path contains the specified path, false otherwise.
	 */
	public boolean contains(Path path) {
		return indexOf(path) != -1;
	}
	
	/**
	 * Returns true if this path contains the specified component, false
	 * otherwise.
	 */
	public boolean contains(String component) {
		return indexOf(component) != -1;
	}
	
	/**
	 * Compares two paths lexicographically.
	 */
	public int compareTo(Path path) {
		return DEFAULT_COMPARATOR.compare(this, path);
	}
	
	/**
	 * Creates a comparator that compares paths lexicographically.
	 */
	public static Comparator<Path> createComprator() {
		return new PathComparator();
	}
	
	/**
	 * Creates a comparator that compares paths lexicographically but with the
	 * specified component comparator.
	 */
	public static Comparator<Path> createComparator(
			Comparator<String> componentComparator) {
		return new PathComparator(componentComparator);
	}
	
	/**
	 * Returns hash code of default string representative of this path.
	 */
	@Override
	public int hashCode() {
		return toString().hashCode();
	}
	
	/**
	 * Returns true if {@code object} is non null instance of {@code Path} and
	 * its components are the same as this path's components, false otherwise.
	 */
	@Override
	public boolean equals(Object object) {
		if (object == null || !(object instanceof Path)) {
			return false;
		}
		return Arrays.equals(components, ((Path) object).components);
	}
	
	/**
	 * Returns path string with the specified separator.
	 * 
	 * @throws IllegalArgumentException
	 *             if {@code separator} is empty.
	 */
	public String toString(String separator) {
		if (separator.length() == 0) {
			throwEmptySeparatorArgument();
		}
		if (components.length == 0) {
			return "";
		}
		
		StringBuilder builder = new StringBuilder();
		int componentCount = components.length;
		
		for (int i = 0; i < componentCount; ++i) {
			String component = components[i];
			
			if (i > 0 && componentCount > 1) {
				builder.append(separator);
			}
			builder.append(component);
		}
		return builder.toString();
	}
	
	/**
	 * Returns a slash separated string of this path.
	 */
	@Override
	public String toString() {
		if (defaultString == null) {
			defaultString = toString(SEPARATOR_SLASH);
		}
		return defaultString;
	}

}
