/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.lang;

import java.lang.reflect.Array;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

/**
 * An object string representative builder that builds programmer friendly
 * string.
 * <p>
 * For example:
 * </p>
 * 
 * <pre>
 * public class ToStringObject {
 * 
 * 	private int id;
 * 	private String name;
 * 
 * 	public ToStringObject(int id, String name) {
 * 		this.id = id;
 * 		this.name = Objects.nonNull(name);
 * 	}
 * 
 * 	public int getId() {
 * 		return id;
 * 	}
 * 
 * 	public String getName() {
 * 		return name;
 * 	}
 * 
 * 	@Override
 * 	public String toString() {
 * 		return StringRepr.fromObject(this)
 * 				.add(&quot;id&quot;, id)
 * 				.add(&quot;name&quot;, name)
 * 				.toString();
 * 	}
 * }
 * 
 * ...
 * 
 * String str =  new ToStringObject(0, &quot;a&quot;).toString();
 * assertEqual(&quot;ToStringObject[id=0, name=\&quot;a\&quot;]&quot;, str);
 * </pre>
 * 
 * @author <a href="mailto:bachew@gmail.com">Chew Boon Aik</a>
 */
// TODO: javadoc cleanup
public class StringRepr {

	private static final String NULL_STR_REPR = "null";

	private final String typeName;
	private final StringBuilder properties;

	/** Creates an string representative from an object. */
	public static StringRepr fromObject(Object obj) {
		return new StringRepr(obj.getClass().getSimpleName());
	}

	/** Creates an string representative from type name. */
	public static StringRepr fromTypeName(String typeName) {
		return new StringRepr(typeName);
	}

	private StringRepr(String typeName) {
		this.typeName = Objects.nonNull(typeName);
		this.properties = new StringBuilder();
	}

	private StringRepr addRaw(String name, String value) {
		if (properties.length() > 0) {
			properties.append(", ");
		}
		properties.append(name);
		properties.append('=');
		properties.append(value);
		return this;
	}

	/**
	 * Adds a {@code boolean} property.
	 */
	public StringRepr add(String name, boolean bool) {
		return addRaw(Objects.nonNull(name), toString(bool));
	}

	/**
	 * Adds a {@code short} property.
	 */
	public StringRepr add(String name, short n) {
		return addRaw(Objects.nonNull(name), toString(n));
	}

	/**
	 * Adds a {@code int} property.
	 */
	public StringRepr add(String name, int n) {
		return addRaw(Objects.nonNull(name), toString(n));
	}

	/**
	 * Adds a {@code long} property.
	 */
	public StringRepr add(String name, long n) {
		return addRaw(Objects.nonNull(name), toString(n));
	}

	/**
	 * Adds a {@code float} property.
	 */
	public StringRepr add(String name, float n) {
		return addRaw(Objects.nonNull(name), toString(n));
	}

	/**
	 * Adds a {@code double} property.
	 */
	public StringRepr add(String name, double n) {
		return addRaw(Objects.nonNull(name), toString(n));
	}

	/**
	 * Adds a {@code char} property.
	 */
	public StringRepr add(String name, char ch) {
		return addRaw(Objects.nonNull(name), toString(ch));
	}

	/**
	 * Adds a {@code String} property.
	 */
	public StringRepr add(String name, @Nullable String str) {
		return addRaw(Objects.nonNull(name), toString(str));
	}

	/**
	 * Adds a {@code Object} property.
	 */
	public StringRepr add(String name, @Nullable Object obj) {
		return addRaw(Objects.nonNull(name), toString(obj));
	}

	/**
	 * Builds and returns this string representative value.
	 */
	@Override
	public String toString() {
		if (properties.length() == 0) {
			return typeName;
		}

		StringBuilder full = new StringBuilder();

		full.append(typeName);
		full.append('[');
		full.append(properties);
		full.append(']');
		return full.toString();
	}

	/**
	 * Equivalent to calling {@code Boolean.toString(bool)}.
	 */
	public static String toString(boolean bool) {
		return Boolean.toString(bool);
	}

	/**
	 * Equivalent to calling {@code Short.toString(n)}.
	 */
	public static String toString(short n) {
		return Short.toString(n);
	}

	/**
	 * Equivalent to calling {@code Integer.toString(n)}.
	 */
	public static String toString(int n) {
		return Integer.toString(n);
	}

	/**
	 * Equivalent to calling {@code Long.toString(n)}.
	 */
	public static String toString(long n) {
		return Long.toString(n);
	}

	/**
	 * Equivalent to calling {@code Float.toString(n)}.
	 */
	public static String toString(float n) {
		return Float.toString(n);
	}

	/**
	 * Equivalent to calling {@code Double.toString(n)}.
	 */
	public static String toString(double n) {
		return Double.toString(n);
	}

	/**
	 * Returns a programmer friendly string representative of {@code ch}. For
	 * examples,
	 * 
	 * <pre>
	 * assertEqual(&quot;'a'&quot;, StringRepr.toString('a'));
	 * assertEqual(&quot;'\\''&quot;, StringRepr.toString('\''));
	 * assertEqual(&quot;'\\n'&quot;, StringRepr.toString('\n'));
	 * </pre>
	 */
	public static String toString(char ch) {
		String nonDispStr = nonDisplayCharStr(ch, '\'');

		if (nonDispStr != null) {
			return "'" + nonDispStr + "'";
		}
		return "'" + ch + "'";
	}

	private static String nonDisplayCharStr(char ch, char quoteChar) {
		// TODO: determine by code point
		if (ch == '\n') {
			return "\\n";
		}
		if (ch == '\r') {
			return "\\r";
		}
		if (ch == quoteChar) {
			return "\\" + ch;
		}
		return null;
	}

	/**
	 * Returns a programmer friendly string representative of {@code str}. For
	 * examples,
	 * 
	 * <pre>
	 * assertEqual(&quot;null&quot;, StringRepr.toString((String) null));
	 * assertEqual(&quot;\&quot;\\\&quot;\\n\&quot;&quot;, StringRepr.toString(&quot;\&quot;\n&quot;));
	 * assertEqual(&quot;\&quot;abc\&quot;&quot;, StringRepr.toString(&quot;abc&quot;));
	 * </pre>
	 */
	public static String toString(@Nullable String str) {
		if (str == null) {
			return NULL_STR_REPR;
		}

		int length = str.length();

		if (length == 0) {
			return "\"\"";
		}

		StringBuilder strRepr = new StringBuilder();

		strRepr.append('"');

		for (int i = 0; i < length; ++i) {
			char ch = str.charAt(i);
			String nonDispStr = nonDisplayCharStr(ch, '"');

			if (nonDispStr != null) {
				strRepr.append(nonDispStr);
			} else {
				strRepr.append(ch);
			}
		}
		strRepr.append('"');
		return strRepr.toString();
	}

	/**
	 * Returns a programmer friendly string representative of {@code array}.
	 * For examples,
	 * 
	 * <pre>
	 * assertEqual(&quot;null&quot;, StringRepr.toStringArray(null));
	 * assertEqual(&quot;[]&quot;, StringRepr.toStringArray(new String[0]));
	 * assertEqual(&quot;[0, \&quot;a\&quot;]&quot;, StringRepr.toStringArray(new Object[] { 0, &quot;a&quot; }));
	 * </pre>
	 * 
	 * @throws IllegalArgumentException
	 *             if the object argument is not an array
	 */
	public static String toStringArray(@Nullable Object array) {
		if (array == null) {
			return NULL_STR_REPR;
		}
		
		int length = Array.getLength(array);

		if (length == 0) {
			return "[]";
		}

		StringBuilder strRepr = new StringBuilder();
		strRepr.append('[').append(toString(Array.get(array, 0)));

		for (int i = 1; i < length; ++i) {
			if (i > 0) {
				strRepr.append(", ");
			}
			strRepr.append(toString(Array.get(array, i)));
		}
		strRepr.append(']');
		return strRepr.toString();
	}

	/**
	 * Returns a programmer friendly string representative of {@code collection}.
	 * For example:
	 * 
	 * <pre>
	 * assertEqual(&quot;null&quot;, StringRepr.toString((Collection&lt;?&gt;) null));
	 * assertEqual(&quot;[]&quot;, StringRepr.toString(Collections.emptyList()));
	 * assertEqual(&quot;[\&quot;a\&quot;, \&quot;b\&quot;]&quot;, StringRepr.toString(Arrays.asList(&quot;a&quot;, &quot;b&quot;)));
	 * </pre>
	 */
	public static String toString(@Nullable Collection<?> collection) {
		if (collection == null) {
			return NULL_STR_REPR;
		}
		if (collection.isEmpty()) {
			return "[]";
		}

		StringBuilder strRepr = new StringBuilder();
		Iterator<?> itemIter = collection.iterator();

		strRepr.append('[').append(toString(itemIter.next()));

		while (itemIter.hasNext()) {
			strRepr.append(", ");
			strRepr.append(toString(itemIter.next()));
		}
		strRepr.append(']');
		return strRepr.toString();
	}
	
	/**
	 * Returns a programmer friendly string representative of {@code map}. For
	 * example:
	 * 
	 * <pre>
	 * assertEqual(&quot;null&quot;, StringRepr.toString((Map&lt;?, ?&gt;) null));
	 * assertEqual(&quot;{}&quot;, StringRepr.toString(Collections.emptyMap()));
	 * assertEqual(&quot;{\&quot;a\&quot;=0}&quot;, StringRepr.toString(Collections.singletonMap(&quot;a&quot;, 0)));
	 * </pre>
	 */
	public static <K, V> String toString(@Nullable Map<K, V> map) {
		if (map == null) {
			return NULL_STR_REPR;
		}
		if (map.isEmpty()) {
			return "{}";
		}

		StringBuilder strRepr = new StringBuilder();
		Iterator<Map.Entry<K, V>> entryIter = map.entrySet().iterator();
		Map.Entry<K, V> entry = entryIter.next();

		strRepr.append("{");
		strRepr.append(toString(entry.getKey()));
		strRepr.append("=");
		strRepr.append(toString(entry.getValue()));

		while (entryIter.hasNext()) {
			entry = entryIter.next();
			strRepr.append(", ");
			strRepr.append(toString(entry.getKey()));
			strRepr.append("=");
			strRepr.append(toString(entry.getValue()));
		}
		strRepr.append("}");
		return strRepr.toString();
	}

	/**
	 * Returns a programmer friendly string representative of {@code obj}.
	 */
	public static String toString(@Nullable Object obj) {
		if (obj == null) {
			return NULL_STR_REPR;
		}
		if (obj instanceof String) {
			return toString((String) obj);
		}
		if (obj.getClass().isArray()) {
			return toStringArray(obj);
		}
		if (obj instanceof Collection) {
			return toString((Collection<?>) obj);
		}
		if (obj instanceof Map) {
			return toString((Map<?, ?>) obj);
		}
		return obj.toString();
	}

}
