/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.resource;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;

import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;
import org.apithefire.util.lang.StringRepr;

class FileResource implements Resource {
	
	private final File file;
	
	public FileResource(File file) {
		this.file = Objects.nonNull(file);
	}
	
	public File getFile() {
		return file;
	}
	
	public boolean isDirectory() {
		return file.isDirectory();
	}

	public Collection<String> list() {
		String[] fileNames = file.list();
		
		if (fileNames == null) {
			return Collections.emptyList();
		}
		
		Collection<String> resourceNames = new ArrayList<String>(fileNames.length);
		
		for (String fileName : fileNames) {
			resourceNames.add(fileName);
		}
		return resourceNames;
	}

	public Resource getResource(Path path) {
		if (path.isEmpty()) {
			return this;
		}
		
		File currentFile = file;
		
		// XXX: bad performance
		for (int i = 0; i < path.length(); ++i) {
			String pathComp = path.componentAt(i);
			
			if (pathComp.equals(".") || pathComp.equals("..")) {
				throw new ResourceNotFoundException(this, path);
			}
			currentFile = new File(currentFile, pathComp);
		}
		if (!currentFile.exists()) {
			throw new ResourceNotFoundException(this, path);
		}
		return new FileResource(currentFile);
	}
	
	public boolean isFile() {
		return file.isFile();
	}
	
	public Date getFileModifiedTime() {
		return new Date(file.lastModified());
	}
	
	public long getFileSize() {
		long length = file.length();
		
		return length > 0 ? length : 0;
	}

	public InputStream openFile() {
		try {
			return new FileInputStream(file);
		} catch (FileNotFoundException e) {
			throw new ResourceException(e);
		}
	}
	
	@Override
	public String toString() {
		return StringRepr.fromObject(this).add("file", file).toString();
	}

}
