/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.resource;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;

import org.apithefire.util.lang.Builder;
import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;

public class FileResourceBuilder implements Builder {
	
	class ResourceImpl implements Resource {

		private Date modifiedTime;
		private long size;
		private FileContent content;

		private ResourceImpl() {
		}

		public boolean isDirectory() {
			return false;
		}

		public Collection<String> list() {
			return Collections.emptyList();
		}

		public Resource getResource(Path path) {
			throw new ResourceNotFoundException(this, path);
		}

		public boolean isFile() {
			return true;
		}

		public Date getFileModifiedTime() {
			return modifiedTime;
		}

		public long getFileSize() {
			return size;
		}

		public InputStream openFile() {
			return content.openFile();
		}
		
	}
	
	class BytesContent implements FileContent {

		private byte[] bytes;
		
		public BytesContent(byte[] bytes) {
			this.bytes = Objects.nonNull(bytes);
		}
		
		public InputStream openFile() {
			return new ByteArrayInputStream(bytes);
		}
		
	}
	
	class ResourceContent implements FileContent {
		
		private Resource resource;
		
		public ResourceContent(Resource resource) {
			this.resource = Objects.nonNull(resource);
		}
		
		public InputStream openFile() {
			return resource.openFile();
		}
		
	}
	
	private Date modifiedTime;
	private long size = -1;
	private FileContent content;
	
	protected FileResourceBuilder() {
	}
	
	public FileResourceBuilder setModifiedTime(Date modifiedTime) {
		this.modifiedTime = modifiedTime;
		return this;
	}
	
	public FileResourceBuilder setSize(long size) {
		if (size < -1) {
			throw new IllegalArgumentException("size < -1");
		}
		this.size = size;
		return this;
	}
	
	public FileResourceBuilder setContent(String string) {
		content = new BytesContent(string.getBytes());
		return this;
	}
	
	public FileResourceBuilder setContent(byte[] bytes) {
		content = new BytesContent(bytes);
		return this;
	}
	
	public FileResourceBuilder setContent(Resource resource) {
		content = new ResourceContent(resource);
		return this;
	}

	public Resource build() {
		ResourceImpl resource = new ResourceImpl();
		
		resource.modifiedTime = modifiedTime;
		resource.size = size;
		resource.content = content;
		return resource;
	}

}
