/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.resource;

import java.io.InputStream;
import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.Enumeration;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;
import org.apithefire.util.lang.StringRepr;

class JarResource implements Resource {
	
	protected static class EmptyIsMaxStringComparator implements Comparator<String> {
		public int compare(String string1, String string2) {
			int length1 = string1.length();
			int length2 = string2.length();
			
			if (length1 == 0 || length2 == 0) {
				if (length1 == length2) {
					return 0;
				}
				return length1 == 0 ? 1 : -1;
			}
			return string1.compareTo(string2);
		}	
	}
	
	private static final Comparator<Path> PATH_COMPARATOR
		= Path.createComparator(new EmptyIsMaxStringComparator());
	
	private final JarFile jarFile;
	private final SortedMap<Path, JarEntry> jarEntryMap;
	private final Path path;
	
	public JarResource(JarFile file) {
		this(file, jarEntryMap(file), Path.EMPTY_PATH);
	}
	
	private static SortedMap<Path, JarEntry> jarEntryMap(JarFile file) {
		SortedMap<Path, JarEntry> entryMap = new TreeMap<Path, JarEntry>(
				PATH_COMPARATOR);
		Enumeration<JarEntry> entryEnm = file.entries();
		
		while (entryEnm.hasMoreElements()) {
			JarEntry iterEntry = entryEnm.nextElement();
			Path iterPath = Path.split(iterEntry.getName()).trim();
			
			entryMap.put(iterPath, iterEntry);
		}
		return entryMap;
	}
	
	private JarResource(JarFile jarFile, SortedMap<Path, JarEntry> jarEntryMap,
			Path path) {
		this.jarFile = Objects.nonNull(jarFile);
		this.jarEntryMap = jarEntryMap;
		this.path = path;
	}
	
	public boolean isDirectory() {
		return true;
	}

	public Collection<String> list() {
		Set<String> names = new TreeSet<String>();
		
		for (Path jarEntryPath : jarEntryMap.keySet()) {
			if (path.length() < jarEntryPath.length()) {
				String name = jarEntryPath.componentAt(path.length());
				
				names.add(name);
			}
		}
		return names;
	}

	public Resource getResource(Path path) {
		if (path.isEmpty()) {
			return this;
		}
		
		Path fromPath = this.path.concat(path);
		Path toPath = fromPath.concat(new Path(""));

		SortedMap<Path, JarEntry> subJarEntryMap = jarEntryMap.subMap(
				fromPath, toPath);
		
		JarEntry contentJarEntry = subJarEntryMap.get(fromPath);
		
		if (contentJarEntry != null
				&& !contentJarEntry.isDirectory()
				&& subJarEntryMap.size() == 1) {
			return new ContentJarResource(jarFile, contentJarEntry);
		}
		if (subJarEntryMap.isEmpty()) {
			throw new ResourceNotFoundException();
		}
		return new JarResource(jarFile, subJarEntryMap, fromPath);
	}
	
	public boolean isFile() {
		return false;
	}

	public Date getFileModifiedTime() {
		return null;
	}

	public long getFileSize() {
		return 0;
	}

	public InputStream openFile() {
		// TODO: throw NotAFileException?
		throw new ResourceException("Can not open "
				+ toString() + " as a file.");
	}
	
	@Override
	public String toString() {
		return StringRepr.fromObject(this)
				.add("jarFile", jarFile)
				.add("path", path)
				.toString();
	}

}
