/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.resource;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;

class MultiResource implements Resource {
	
	private final Collection<Resource> resources;
	
	public MultiResource(Collection<Resource> resources) {
		this.resources = new ArrayList<Resource>(resources);
		// TODO: lazy init
	}

	public boolean isDirectory() {
		return findFirstDirectory() != null;
	}
	
	private Resource findFirstDirectory() {
		for (Resource resource : resources) {
			if (resource.isDirectory()) {
				return resource;
			}
		}
		return null;
	}

	public Collection<String> list() {
		Set<String> names = new TreeSet<String>();
		
		for (Resource resource : resources) {
			if (resource.isDirectory()) {
				names.addAll(resource.list());
			}
		}
		return names;
	}

	public Resource getResource(Path path) {
		List<Resource> resources = getResources(path);
		
		if (resources.size() == 1) {
			return resources.get(0);
		}
		return new MultiResource(resources);
	}
	
	private List<Resource> getResources(Path path) {
		Objects.nonNull(path);
		
		List<Resource> foundResources = new ArrayList<Resource>();
		
		for (Resource resource : resources) {
			if (resource.isDirectory()) {
				try {
					foundResources.add(resource.getResource(path));
				} catch (ResourceNotFoundException e) {
					continue;
				}
			}
		}
		if (foundResources.isEmpty()) {
			throw new ResourceNotFoundException(this, path);
		}
		return foundResources;
	}

	public boolean isFile() {
		return findFirstFile() != null;
	}
	
	private Resource findFirstFile() {
		for (Resource resource : resources) {
			if (resource.isFile()) {
				return resource;
			}
		}
		return null;
	}

	public Date getFileModifiedTime() {
		Resource file = findFirstFile();
		
		if (file == null) {
			// TODO: not a file, what to do?
			return null;
		}
		return file.getFileModifiedTime();
	}

	public long getFileSize() {
		Resource file = findFirstFile();
		
		if (file == null) {
			// TODO: not a file, what to do?
			return -1;
		}
		return file.getFileSize();
	}

	public InputStream openFile() {
		Resource file = findFirstFile();
		
		if (file == null) {
			// TODO: throw not a file
		}
		return file.openFile();
	}

}
