/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.resource;

import java.io.File;
import java.io.IOException;
import java.net.JarURLConnection;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.jar.JarFile;

import org.apithefire.util.io.RuntimeIoException;
import org.apithefire.util.lang.Path;
import org.apithefire.util.lang.Strings;
import org.apithefire.util.lang.Systems;

public class Resources {
	
	private static final EmptyDirectoryResource EMPTY_DIR = new EmptyDirectoryResource();

	private Resources() {
	}
	
	public static FileResourceBuilder newFileBuilder() {
		return new FileResourceBuilder();
	}
	
	public static DirectoryResourceBuilder newDirectoryBuilder() {
		return new DirectoryResourceBuilder();
	}
	
	public static Resource fromClassPath() {
		List<String> classPath = Systems.getClassPath();
		
		if (classPath.isEmpty()) {
			return EMPTY_DIR;
		}
		if (classPath.size() == 1) {
			return fromClassPathComponent(classPath.get(0));
		}
		
		List<Resource> resources = new ArrayList<Resource>();
		
		for (String classPathComp : classPath) {
			resources.add(fromClassPathComponent(classPathComp));
		}
		return new MultiResource(resources);
	}
	
	private static Resource fromClassPathComponent(String component) {
		try {
			return fromUrl(new URL(component));
		} catch (MalformedURLException e) {
			// continue
		}
		
		File file = new File(component);
		
		if (file.isDirectory()) {
			return new FileResource(file);
		}
		try {
			return new JarResource(new JarFile(file));
		} catch (IOException e) {
			// continue
		}
		return new FileResource(file);
	}
	
	public static Resource relativeToClass(Class<?> clazz) {
		return fromClassPath().getResource(Path.split(clazz.getPackage()));
	}
	
	public static Resource fromJarFile(File jarFile) {
		try {
			return new JarResource(new JarFile(jarFile));
		} catch (IOException e) {
			throw new RuntimeIoException();
		}
	}
	
	private static Resource fromUrl(URL url) {
		String protocol = url.getProtocol();
		
		if ("file".equals(protocol)) {
			return fromFileUrl(url);
		}
		if ("jar".equals(protocol)) {
			return fromJarUrl(url);
		}
		throw new ResourceException("Unrecognized protocol in url " + url);
	}
	
	private static Resource fromFileUrl(URL url) {
		// TODO: Are all URLs URI? Especially the file in jar URL
		File file = new File(URI.create(url.toString()));
		
		return new FileResource(file);
	}
	
	private static Resource fromJarUrl(URL url) {
		try {
			JarURLConnection connection = (JarURLConnection)
					url.openConnection();
			
			connection.setUseCaches(false);
			
			JarFile jarFile = connection.getJarFile();
			JarResource resource = new JarResource(jarFile);
			String entryName = Strings.nullIfEmpty(connection.getEntryName());
			
			if (entryName == null) {
				return resource;
			}
			return resource.getResource(Path.split(entryName));
		} catch (IOException e) {
			throw new RuntimeIoException(e);
		}
	}
	
	public static Resource fromFile(File file) {
		return new FileResource(file);
	}
	
}
