/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.test;

import org.apithefire.util.lang.Nullable;
import org.apithefire.util.lang.Objects;

/**
 * Static assert methods.
 * 
 * @author <a href="mailto:bachew@gmail.com">Chew Boon Aik</a>
 */
public class Assertion {
	
	private Assertion() {
	}
	
	/**
	 * Throws AssertionError if {@code b} is {@code false}.
	 */
	public static void assertTrue(boolean b) {
		if (!b) {
			throw new AssertionError();
		}
	}

	/**
	 * Throws AssertionError if {@code b} is {@code true}.
	 */
	public static void assertFalse(boolean b) {
		if (b) {
			throw new AssertionError();
		}
	}
	
	private static boolean equal(@Nullable Object obj1, @Nullable Object obj2) {
		return obj1 == null ? obj2 == null : obj1.equals(obj2);
	}

	/**
	 * Throws AssertionError if {@code obj1} and {@code obj2} are not
	 * equal.
	 */
	public static void assertEqual(@Nullable Object obj1, @Nullable Object obj2) {
		if (!Objects.equal(obj1, obj2)) {
			throw new AssertionError(obj1 + " is not equal to " + obj2 + ".");
		}
	}

	/**
	 * Throws AssertionError if {@code obj1} and {@code obj2} are equal.
	 */
	public static void assertNotEqual(@Nullable Object obj1, @Nullable Object obj2) {
		if (equal(obj1, obj2)) {
			throw new AssertionError(obj1 + " is equal to " + obj2 + ".");
		}
	}

	/**
	 * Throws AssertionError if {@code obj} is not null.
	 */
	public static void assertNull(@Nullable Object obj) {
		if (obj != null) {
			throw new AssertionError();
		}
	}

	/**
	 * Throws AssertionError if {@code obj} is null.
	 */
	public static void assertNotNull(@Nullable Object obj) {
		if (obj == null) {
			throw new AssertionError();
		}
	}
	
	/**
	 * Creates an {@link ExceptionAssertion} for exception assertion.
	 */
	public static ExceptionAssertion assertException() {
		return new ExceptionAssertion();
	}

}
