/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.test;

import java.util.ArrayList;
import java.util.List;

import org.apithefire.util.lang.Objects;

/**
 * An exception checker. 
 * 
 * @author <a href="mailto:bachew@gmail.com">Chew Boon Aik</a>
 */
public class ExceptionAssertion {

	private final List<ExceptionChecker> checkers = new ArrayList<ExceptionChecker>();
	
	protected ExceptionAssertion() {
	}
	
	/**
	 * Adds a checker that will be called to check for exception thrown.
	 */
	public ExceptionAssertion checker(ExceptionChecker checker) {
		checkers.add(Objects.nonNull(checker));
		return this;
	}

	/**
	 * Adds a exception checker that check if the exception thrown is instance
	 * of the specified type.
	 * 
	 * @see #checker
	 */
	public ExceptionAssertion type(final Class<? extends Throwable> type) {
		if (type == null) {
			throw new NullPointerException();
		}
		checker(new ExceptionChecker() {
			public void check(Throwable exception) {
				if (!type.isAssignableFrom(exception.getClass())) {
					throw new AssertionError(
							"Expected exception " + type.getName()
							+ " not thrown but was "
							+ exception.getClass().getName() + ".");
				}
			}
		});
		return this;
	}
	
	/**
	 * Adds a exception checker that check if the exception thrown contains the
	 * specified string case insensitively.
	 * 
	 * @see #checker
	 */
	public ExceptionAssertion messageContains(final String string) {
		if (string == null) {
			throw new NullPointerException();
		}
		checker(new ExceptionChecker() {
			public void check(Throwable exception) {
				String message = exception.getMessage();
				
				if (message == null
						|| !message.toUpperCase().contains(string.toUpperCase())) {
					throw new AssertionError(
							"Exception message does not contains: " + string);
				}
			}
		});
		return this;
	}
	
	/**
	 * Runs the specified runnable and check for exception thrown with added
	 * exception checkers.
	 * 
	 * @throws AssertionError
	 *             if exception is not thrown or any one of the added checkers
	 *             throws AssertionError
	 */
	public void in(Runnable runnable) {
		Throwable caught = null;
		try {
			runnable.run();
		} catch (Throwable e) {
			caught = e;
		}
		if (caught == null) {
			throw new AssertionError("No exception thrown.");
		}
		for (ExceptionChecker checker : checkers) {
			checker.check(caught);
		}
	}
	
}
