/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package org.apithefire.util.test;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apithefire.util.lang.Classes;
import org.apithefire.util.lang.Objects;

class TestClassFactory {
	
	private static class TestClassImpl implements TestClass {
		
		private final Class<? extends Test> testClass;
		private final Set<TestClass> dependencies;
		
		public TestClassImpl(Class<? extends Test> testClass) {
			this.testClass = Objects.nonNull(testClass);
			this.dependencies = new HashSet<TestClass>();
		}

		public Set<TestClass> getDependencies() {
			return Collections.unmodifiableSet(dependencies);
		}
		
		private void addDependency(TestClass dependency) {
			dependencies.add(Objects.nonNull(dependency));
		}

		public Test newTest() {
			return Classes.newInstance(testClass);
		}
		
		@Override
		public boolean equals(Object obj) {
			if (obj == null || !(obj instanceof TestClassImpl)) {
				return false;
			}
			return testClass.equals(((TestClassImpl) obj).testClass);
		}
		
		@Override
		public int hashCode() {
			return testClass.hashCode();
		}
		
	}

	public TestClass createTestClass(Class<? extends Test> clazz) {
		return createTestClass(clazz, new HashSet<Class<? extends Test>>(),
				new ArrayList<Class<? extends Test>>());
	}
	
	private TestClass createTestClass(Class<? extends Test> clazz,
			Set<Class<? extends Test>> visitedClassSet,
			List<Class<? extends Test>> visitedClassList) {
		if (visitedClassSet.contains(Objects.nonNull(clazz))) {
			StringBuilder message = new StringBuilder();
			
			for (Class<? extends Test> visitedClass : visitedClassList) {
				message.append(visitedClass.getSimpleName());
				message.append(" -> ");
			}
			message.append(clazz.getSimpleName());
			throw new CircularDependencyException(message.toString());
		}
		visitedClassSet.add(clazz);
		visitedClassList.add(clazz);
		
		TestClassImpl testClass = new TestClassImpl(clazz);
		Test.Depends dependsAnn = clazz.getAnnotation(Test.Depends.class);
		
		if (dependsAnn != null && dependsAnn.value().length > 0) {
			for (Class<? extends Test> dependClass : dependsAnn.value()) {
				TestClass dependTestClass = createTestClass(dependClass,
						visitedClassSet, visitedClassList);
				
				visitedClassSet.add(clazz);
				testClass.addDependency(dependTestClass);
			}
		}
		visitedClassSet.remove(clazz);
		visitedClassList.remove(visitedClassList.size() - 1);
		return testClass;
	}
	
}
